package files::module;
use strict;
use warnings;

sub one {
    # this
    return 1;
}
sub two {
    # this
    return 1;
}
sub three {
    #this
    return 1;
}
sub four {
    #this
    return 1;
}
sub five {
    #this
    return 1;
}
1;