use warnings;
use strict;
# sample data file
sub one {
    # one
sub one_inner {
        sub one_inner_two {
            # nothing 
        }
        return 0;
    }
    # two
    # three
}
sub two {
    my $x = 0;
    my $y = 0;
    my $result = function( $x, $y );
}
sub three {

    # four
    # five
    my $str = 'this';
    # six

}
sub four {
    # hello
    # there
    # wonderful
    # world
}
sub function {
    my ( $x, $y ) = @_;
    return $x * $y;
}
    sub five {
        my $str = this;
}
sub six {
    my $str = this;
}
sub seven {
    my $str = this;
    }
sub eight {
    my $str = this;
}
